#!/usr/bin/python
#
# HD44780 LCD Class for Raspberry Pi
#
# Developed by: AndyPi (http://andypi.co.uk/)
# Based on work by : Matt Hawkins + Texy
#
# Date   : 27/03/2013
#
# Updated RSS Feed ( Update_News(URL) Will update the News Array which can be accessed with News[0] ( Number 1 ), News[1] etc.

import time, sys, re, os
import subprocess, signal
from datetime import datetime


Delay = 300
News = []
Feed = 'feeds.bbci.co.uk/news/rss.xml?edition=uk'


def Update_News(Url):
	News = []
	Args = ['curl' , Url]
	Proc = subprocess.Popen(Args, stdout=subprocess.PIPE)
	Out = Proc.stdout.read()
	Find = re.findall("<title>([\S ]*)</title>", Out)
	Find.pop(0)
	Find.pop(0)

	for A in Find:
		if A[:7] == "VIDEO: " or A[:7] == "AUDIO: " or A[:13] == "In pictures: " or A[:17] == "Day in pictures: ":
			A = ""
		else:
			News.append(A)

	return News

class AndyPi_LCD:

  # Define GPIO to LCD mapping
  LCD_RS = 25
  LCD_E  = 24
  LCD_D4 = 23
  LCD_D5 = 17
  LCD_D6 = 27
  LCD_D7 = 22
  LCD_LED = 18

  # Define some device constants
  LCD_WIDTH = 20    # Maximum characters per line
  LCD_CHR = True
  LCD_CMD = False

  LCD_LINE_1 = 0x80 # LCD RAM address for the 1st line
  LCD_LINE_2 = 0xC0 # LCD RAM address for the 2nd line 
  LCD_LINE_3 = 0x94
  LCD_LINE_4 = 0xD4

  # Timing constants
  E_PULSE = 0.00005
  E_DELAY = 0.00005
  
  # import wiringpi class (need to install by ???
  import wiringpi
  #  import GPIO class (need to install sudo pip install rpi-gpio
  import RPi.GPIO as GPIO

  # Setup GPIO 
  GPIO.setmode(GPIO.BCM)       # Use BCM GPIO numbers
  GPIO.setwarnings(False)
  GPIO.setup(LCD_E, GPIO.OUT)  # E
  GPIO.setup(LCD_RS, GPIO.OUT) # RS
  GPIO.setup(LCD_D4, GPIO.OUT) # DB4
  GPIO.setup(LCD_D5, GPIO.OUT) # DB5
  GPIO.setup(LCD_D6, GPIO.OUT) # DB6
  GPIO.setup(LCD_D7, GPIO.OUT) # DB7
  wiringpi.wiringPiSetupGpio()

  	
  def main(self):
    # Send a test text
    self.lcd_init()
    self.led(512)
    if len(sys.argv) > 2:
	Speed = sys.argv[1]
    else:
	Speed = 0.50

    News = Update_News('feeds.bbci.co.uk/news/rss.xml?edition=uk')
    self.scroll_clock(1,"c",Speed," "*20 + News[0], " "*20 + News[1], " "*20 + News[2])
    time.sleep(Delay)
    self.main()

  def lcd_init(self):
    # Initialise display
    self.lcd_byte(0x33,self.LCD_CMD)  # init
    self.lcd_byte(0x32,self.LCD_CMD)  # init
    self.lcd_byte(0x28,self.LCD_CMD)  # 2 line 5x7 matrix
    self.lcd_byte(0x0C,self.LCD_CMD)  # turn cursor off (0x0E to enable)
    self.lcd_byte(0x06,self.LCD_CMD)  # shift cursor right
    self.lcd_byte(0x01,self.LCD_CMD)  # clear display

  def cls(self):
    self.lcd_byte(0x01,self.LCD_CMD)  # clear display
    self.led(0) # turn off backlight

  def static_text(self,line,just,message):
    # set justifitication
    if just=="l":
      message = message.ljust(self.LCD_WIDTH," ")

    elif just=="r":
      message = message.rjust(self.LCD_WIDTH," ")

    elif just=="c":
      message = message.center(self.LCD_WIDTH," ")
    
    time.sleep(0.01)
    # set which line to print to
    if line==1:line=self.LCD_LINE_1
    elif line==2:line=self.LCD_LINE_2
    elif line==3:line=self.LCD_LINE_3
    elif line==4:line=self.LCD_LINE_4
    self.lcd_byte(line,self.LCD_CMD)
    # send string to display
    for i in range(self.LCD_WIDTH):
      self.lcd_byte(ord(message[i]),self.LCD_CHR)
    
  def scroll(self,line,s, scrollmsg):
    # scroll display
    scrollmsg = (scrollmsg+"   |   ") * 200
    z=0
    try:
      while z < len(scrollmsg):
        self.static_text(line,"l", scrollmsg[z:(z+self.LCD_WIDTH)])
        time.sleep(s)
        z=z+1
        if z==len(scrollmsg):
          z=0
    except KeyboardInterrupt:
      self.cls()

  def clock(self,line,just):
    if line==1:line=self.LCD_LINE_1
    elif line==2:line=self.LCD_LINE_2
    elif line==3:line=self.LCD_LINE_3
    elif line==4:line=self.LCD_LINE_4
    try:
      while True:
        self.static_text(line,just,(datetime.now().strftime('%H:%M:%S')))
        #(datetime.now().strftime('%a %d %b %y'))
        time.sleep(1)
    except KeyboardInterrupt:
        self.cls()


  def scroll_clock(self,line,just,s, scrollmsg1="", scrollmsg2="", scrollmsg3=""):
    # scroll display
    z=0
    if line==1:
	line_2=2
    elif line==2:
	line_2=3
    if scrollmsg2:
	line_3 = 3
    if scrollmsg3:
	line_4 = 4
    try:
      while z < len(scrollmsg1):
        self.static_text(line_2,"l", scrollmsg1[z:(z+self.LCD_WIDTH)])
	if scrollmsg2:
	        self.static_text(line_3,"l", scrollmsg2[z:(z+self.LCD_WIDTH)])
		if len(scrollmsg2) > len(scrollmsg1):
			scrollmsg1 = scrollmsg1 + " " * (len(scrollmsg2) - len(scrollmsg1))

		if len(scrollmsg3) > len(scrollmsg1) and len(scrollmsg3) > len(scrollmsg2):
			scrollmsg1 = scrollmsg1 + " " * (len(scrollmsg3) - len(scrollmsg1))
	if scrollmsg3:
	        self.static_text(line_4,"l", scrollmsg3[z:(z+self.LCD_WIDTH)])
        x=datetime.now().strftime('%a %d %b   %H:%M')
        self.static_text(line,"l",x)
        time.sleep(Delay)
        z=z+1
        if z==len(scrollmsg1):
          self.main()
    except KeyboardInterrupt:
      self.cls()

  def led(self,led_value):
    if led_value==0:
      self.wiringpi.pinMode(self.LCD_LED, 1) # turn off LED
    else:   
      self.wiringpi.pinMode(self.LCD_LED, 2) # LED set up as PWM
      self.wiringpi.pwmWrite(self.LCD_LED, led_value)

  def lcd_byte(self,bits, mode):
    self.GPIO.output(self.LCD_RS, mode) # RS

    self.GPIO.output(self.LCD_D4, False)
    self.GPIO.output(self.LCD_D5, False)
    self.GPIO.output(self.LCD_D6, False)
    self.GPIO.output(self.LCD_D7, False)
    if bits&0x10==0x10:
      self.GPIO.output(self.LCD_D4, True)
    if bits&0x20==0x20:
      self.GPIO.output(self.LCD_D5, True)
    if bits&0x40==0x40:
      self.GPIO.output(self.LCD_D6, True)
    if bits&0x80==0x80:
      self.GPIO.output(self.LCD_D7, True)

    time.sleep(self.E_DELAY)    
    self.GPIO.output(self.LCD_E, True)  
    time.sleep(self.E_PULSE)
    self.GPIO.output(self.LCD_E, False)  
    time.sleep(self.E_DELAY)      

    self.GPIO.output(self.LCD_D4, False)
    self.GPIO.output(self.LCD_D5, False)
    self.GPIO.output(self.LCD_D6, False)
    self.GPIO.output(self.LCD_D7, False)
    if bits&0x01==0x01:
      self.GPIO.output(self.LCD_D4, True)
    if bits&0x02==0x02:
      self.GPIO.output(self.LCD_D5, True)
    if bits&0x04==0x04:
      self.GPIO.output(self.LCD_D6, True)
    if bits&0x08==0x08:
      self.GPIO.output(self.LCD_D7, True)

    time.sleep(self.E_DELAY)    
    self.GPIO.output(self.LCD_E, True)  
    time.sleep(self.E_PULSE)
    self.GPIO.output(self.LCD_E, False)  
    time.sleep(self.E_DELAY)   



def shutdown(signum, frame):
	lcd.led(0)
	lcd.cls()
	sys.exit()
	print("Shutdown Gracefully.")

signal.signal(signal.SIGTERM, shutdown)
signal.signal(signal.SIGINT , shutdown)

if __name__ == '__main__':
	if len(sys.argv) > 1:
		Delay = float(sys.argv[1]) / 10
		print "DELAY:", Delay
	else:
		Delay = 20
	Update_News('feeds.bbci.co.uk/news/rss.xml?edition=uk')
	lcd=AndyPi_LCD()
	lcd.main()
